function Par=MaxKcut(B,I)
% equi-max cut
% min  tr (-L) X
% s.t. Xe = m           (1)
%      diag(X) = e      (2)
%      X >=(SDP) 0      (3)
%      X >=(element) 0  (4)
%
%      note that the qusi-rank-constraint mI-X>=0 is entailed by (1)(2)(4)
%
% input: the adjacenc matrix B
%        the number of sub-graph I
% output: the partition Par (Par.x: matrix representation, Par.l: I-nary labels
%         the cutweight CW
%         the attained cutweight CWa
%         the relaxed solution X (in case other rounding methods will be used)
%         SDP info

L = 20;   % number of randomized roundings
n=length(B);
e=ones(n,1);
W=-(spdiags(B*e,0,n,n)-B); 

m=n*(n-1)/2;
A_linear=sparse([],[],[],n^2,m,m);
count = 0; 

% non-negative constraints -------------------
for k= 1:n 
  for j = k+1:n
    count = count+1; 
    A_linear(:,count)=vec(sparse(k,j,1,n,n));
  end
end
At=[spdiags(-ones(m,1),0,m,m);A_linear];

% diagonal constraints -----------------------
for k=1:n
  At(:,count+k)=[sparse(m,1);vec(sparse(k,k,1,n,n))];
end

c=[sparse(m,1);vec(W)];
b=[sparse(m,1);e];
K.l=[m];
K.s=[n];

% now plus equipartition constraints
% supposed I equi-size components and each 
% component is of size Im
%%%%% I = 4;
Im = n/I;   % size of each sub-graph
me = Im*e;

count1=count+n;
for k=1:n
  temp=sparse([],[],[],n,n,n);
  temp(:,k)=ones(n,1);
  At(:,count1+k)=[sparse(m,1);vec(temp)];
end

c=[sparse(m,1);vec(W)];
b=[sparse(m,1);e;me];
K.l=[m];
K.s=[n];

[x,y,info]=sedumi(At,b,c,K);
X=mat(x(m+1:length(x)));
CW=-trace(W*X);


% randomized rounding:
[e1,e2]=eig(X);
[lam,order]=sort(diag(abs(e2)));
% if X is SDP, then sqrt(e2) defined
Y=e1*sqrt(e2);  
V=Y';  % each column of V is a point


% randomized method for rounding 
for l=1:L
  for k=1:I
    z(:,k)=randn(n,1);
  end

  round1=Y*z;
  [value1,id1]=max(round1,[],2);

  ans1=zeros(n,I);
  for i=1:n
    ans1(i,id1(i))=1;
  end

  Cut1(l)=-trace(ans1'*W*ans1);
  Ans{l}=ans1;
end


% solutions:
Par.CW=CW;
[Par.CWr(1),imax]=max(Cut1);
Par.x{1}=Ans{imax};
Par.X=X;
[temp,Par.label{1}]=max(Par.x{1},[],2);
Par.info = info;
Par.CWa(1)=-Inf;


% ------------------------------------------------
% K-means methods for rounding
label=round(rand(n,1));

% original Y
V=Y';  % each column of V is a point, full rank
[Vv,X1,Vn]=pca(cat(2,V,-V),I);  % Vv is the principle directions
Y0=Y*Vv; 
repeats=20;
[Quality, Edistortion, Adistortion, Rdistortion, separation, ...
         Rdistortion1, invScatter, SOLUTIONS] = ...
         new_kmeans_equi(Y0, I, label, repeats);
%[dist,best]=min(Edistortion);
%id2=SOLUTIONS(best,:);
for k=1:repeats
  id2=SOLUTIONS(k,:);
  ans2{k}=zeros(n,I);
  for i=1:n
    ans2{k}(i,id2(i))=1;
  end
  Cut2(k)=-trace(ans2{k}'*W*ans2{k});
end

[Par.CWa(2),imax]=max(Cut2);
Par.x{2}=ans2{imax};
[temp,Par.label{2}]=max(Par.x{2},[],2);


% rank-I Y
VI=Y(:,order(n-I+1:n))';    % rank I
[VvI,X1I,VnI]=pca(cat(2,VI,-VI),I);  % using the rank-k relaxation
YI0=Y(:,order(n-I+1:n))*VvI; 
repeats=20;
[Quality, Edistortion, Adistortion, Rdistortion, separation, ...
         Rdistortion1, invScatter, SOLUTIONS] = ...
         new_kmeans_equi(YI0, I, label, repeats);
%[dist,best]=min(Edistortion);
%id2=SOLUTIONS(best,:);

for k=1:repeats
  id3=SOLUTIONS(k,:);
  ans3{k}=zeros(n,I);
  for i=1:n
    ans3{k}(i,id3(i))=1;
  end
  Cut3(k)=-trace(ans3{k}'*W*ans3{k});
end

[Par.CWa(3),imax]=max(Cut3);
Par.x{3}=ans3{imax};
[temp,Par.label{3}]=max(Par.x{3},[],2);


% ------------------------------------------
% proning:

