%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GMF main driver
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function stat=gmf_routine(d_obs,d_coup,prob,n,KK,mm)

G = graph(n,prob);
e=ones(n,1);
W=(spdiags(G*e,0,n,n)-G);

%viewgraph(G);

u0 = (rand(n,1)-0.5)*2*d_obs;% singlten weight
temp = (rand(n,n)*d_coup);
u = (temp+temp').*G/2;       % attractive or repulsive edge weight

stat.G=G;
stat.u=u;
stat.u0=u0;
[tg cliques fill_ins order] = triangulate_max(G);
TW=treewidth(cliques);
stat.TW=TW;

% weight for 0-1 state space ------------------
for i=1:n
  t0(i)=2*u0(i)-2*(sum(u(i,:))+sum(u(:,i)));
end
tt=4*u;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% equivalent weighting scheme for GBP code
Psi=cell(n,n);
Ls=cell(n,1);
for i=1:n
  Ls{i}=[exp(u0(i));exp(-u0(i))];
  for j=1:n
    if G(i,j)~=0
      Psi{i,j}=[exp(u(i,j)) exp(-u(i,j)); exp(-u(i,j)) exp(u(i,j))];
    end
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

row=1;
% ----------Exact --------

a=1;
tic;
t=cputime;
[belG,logZ]=exactInference_new(G,Ls,Psi);
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
stat.belG=belG;
stat.logZ=logZ;
%trueBel2=cat(1, belG, (1-belG));
save stat.mat stat -MAT;
mm1=num2str(mm)
filename=strcat('stat',mm1,'.mat');
save(filename,'stat','-MAT');

%[belG]=exactInference(G,Ls,Psi);
%for i=1:n
%  M1(:,i)=belG{i};
%end


for i=1:n
  for j=1:n
    inv_u(i,j)=u(i,j)*(u(i,j)==0)+(1/(u(i,j)+1e-100))*(u(i,j)~=0);
  end
end
inv_u=abs(inv_u);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
repeat=10;

% ----------MF------------
a=2;
tic;
t=cputime;
for i=1:repeat
  [Mar0{i},logZ0(i),temp_b0(i)]=MF(G,u0,u);
  temp_err0(i)=sum(sum(abs((Mar0{i}-belG))))/(2*n);
end
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
imax=argmax(temp_b0);
bound(row,a)=temp_b0(imax);
err(row,a)=temp_err0(imax);
logzz(row,a)=logZ0(imax);

stat.bound=bound;
stat.err=err;
stat.Mar0=Mar0{imax};
stat.logzz=logzz;
save stat_new.mat stat -MAT;




% ----------GMF-----------
row=2;

K=KK(row-1);
a=1;
tic;
t=cputime;
% Graph partition: MaxCut
Par=MaxKcut(G,K);
[Maxcut,maxI]=max(Par.CWa);
Y=Par.x{maxI};
for i=1:repeat
  [Mar1{i},logZ1(i),temp_b1(i)]=MF_block(G,Y,u0,u);
  temp_err1(i)=sum(sum(abs((Mar1{i}-belG))))/(2*n);
end
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
stat.MaxC(1)=Par.CW;
stat.MaxC(2)=Maxcut;
imax=argmax(temp_b1);
bound(row,a)=temp_b1(imax);
err(row,a)=temp_err1(imax);
logzz(row,a)=logZ1(imax);

stat.bound=bound;
stat.err=err;
stat.logzz=logzz;
stat.Mar1=Mar1{imax};
save stat_new.mat stat -MAT;

%-----

a=2;
tic;
t=cputime;
% Graph partition: MinCut
Par=MinKcut(G,K);
[Mincut,minI]=min(Par.CWa);
Y=Par.x{minI};
for i=1:repeat
  [Mar2{i},logZ2(i),temp_b2(i)]=MF_block(G,Y,u0,u);
  temp_err2(i)=sum(sum(abs((Mar2{i}-belG))))/(2*n);
end
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
stat.MinC(1)=Par.CW;
stat.MinC(2)=Mincut;
imax=argmax(temp_b2);
bound(row,a)=temp_b2(imax);
err(row,a)=temp_err2(imax);
logzz(row,a)=logZ2(imax);

stat.bound=bound;
stat.err=err;
stat.logzz=logzz;
stat.Mar2=Mar2{imax};
save stat_new.mat stat -MAT;



%----------
a=3;
tic;
t=cputime;
% random partition
%for i=1:repeat
  M=n/K;
  Yp=zeros(n,K);
  l1=randperm(n);
  for i=1:K
    for m=1:M
      Yp(l1((i-1)*M+m),i)=1;
    end
  end
  Rcut=trace(Yp'*W*Yp);
for i=1:repeat
  [Mar3{i},logZ3(i),temp_b3(i)]=MF_block(G,Yp,u0,u);
  temp_err3(i)=sum(sum(abs((Mar3{i}-belG))))/(2*n);
end
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
stat.RC(1)=Rcut;
imax=argmax(temp_b3);
bound(row,a)=temp_b3(imax);
err(row,a)=temp_err3(imax);
logzz(row,a)=logZ3(imax);

stat.bound=bound;
stat.err=err;
stat.logzz=logzz;
stat.Mar3=Mar3{imax};
save stat_new.mat stat -MAT;

%--------------------------------------------------------------
% ----------GMF based on weight-----------

a=4;
tic;
t=cputime;
% Graph partition: MaxCut
Par=MaxKcut(abs(u),K);
[Maxcut,maxI]=max(Par.CWa);
Y=Par.x{maxI};
for i=1:repeat
  [Mar11{i},logZ11(i),temp_b11(i)]=MF_block(G,Y,u0,u);
  temp_err11(i)=sum(sum(abs((Mar11{i}-belG))))/(2*n);
end
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
stat.MaxC1(1)=Par.CW;
stat.MaxC1(2)=Maxcut;
imax=argmax(temp_b11);
bound(row,a)=temp_b11(imax);
err(row,a)=temp_err11(imax);
logzz(row,a)=logZ11(imax);

stat.bound=bound;
stat.err=err;
stat.logzz=logzz;
stat.Mar4=Mar11{imax};
save stat_new.mat stat -MAT;




% -------------
a=5;
tic;
t=cputime;
% Graph partition: MinCut
Par=MinKcut(abs(u),K);
[Mincut,minI]=min(Par.CWa);
Y=Par.x{minI};
for i=1:repeat
  [Mar21{i},logZ21(i),temp_b21(i)]=MF_block(G,Y,u0,u);
  temp_err21(i)=sum(sum(abs((Mar21{i}-belG))))/(2*n);
end
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
stat.MinC1(1)=Par.CW;
stat.MinC1(2)=Mincut;
imax=argmax(temp_b21);
bound(row,a)=temp_b21(imax);
err(row,a)=temp_err21(imax);
logzz(row,a)=logZ21(imax);

stat.bound=bound;
stat.err=err;
stat.logzz=logzz;
stat.Mar5=Mar21{imax};
save stat_new.mat stat -MAT;

% ----------GMF based on 1/weight-----------


a=6;
tic;
t=cputime;
% Graph partition: MaxCut
Par=MaxKcut(inv_u,K);
[Maxcut,maxI]=max(Par.CWa);
Y=Par.x{maxI};
for i=1:repeat
  [Mar11{i},logZ11(i),temp_b11(i)]=MF_block(G,Y,u0,u);
  temp_err11(i)=sum(sum(abs((Mar11{i}-belG))))/(2*n);
end
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
stat.MaxC2(1)=Par.CW;
stat.MaxC2(2)=Maxcut;
imax=argmax(temp_b11);
bound(row,a)=temp_b11(imax);
err(row,a)=temp_err11(imax);
logzz(row,a)=logZ11(imax);

stat.bound=bound;
stat.err=err;
stat.logzz=logzz;
stat.Mar6=Mar11{imax};
save stat_new.mat stat -MAT;

% -------------
a=7;
tic;
t=cputime;
% Graph partition: MinCut
Par=MinKcut(inv_u,K);
[Mincut,minI]=min(Par.CWa);
Y=Par.x{minI};
for i=1:repeat
  [Mar21{i},logZ21(i),temp_b21(i)]=MF_block(G,Y,u0,u);
  temp_err21(i)=sum(sum(abs((Mar21{i}-belG))))/(2*n);
end
t1=cputime;
stat.Ecpu(row,a)=t1-t;
stat.Etic(row,a)=toc;
stat.MinC2(1)=Par.CW;
stat.MinC2(2)=Mincut;
imax=argmax(temp_b21);
bound(row,a)=temp_b21(imax);
err(row,a)=temp_err21(imax);
logzz(row,a)=logZ21(imax);

stat.bound=bound;
stat.err=err;
stat.logzz=logzz;
stat.Mar7=Mar21{imax};
save stat_new.mat stat -MAT;


mm1=num2str(mm)
filename=strcat('stat_new',mm1,'.mat');
save(filename,'stat','-MAT');



