function [G,cliques,fill_ins,order] = triangulate(G);

% Triangulates g by elimination in order.
%
%%% Murphy's code with ordering heuristic.

MG = G; % original graph
n = length(G);
eliminated = zeros(1,n);
cliques = {};
order = zeros(1,n);

for ii=1:n
  noteliminated = double(~eliminated);
  EG = G & (noteliminated' * noteliminated); % graph with eliminated nodes removed
  UU = find(~eliminated); % uneliminated

  % determines number of neighbours for each node
  numneighbours = sum(EG); 
  UU = find(eliminated==0);
  % determines the nodes with the fewest neighbours
  [dm,indx] = min(numneighbours(UU)); 
  minneighbours = find(numneighbours(UU)==dm);
  % if >1 node tied for min neighbours, use one with max connected neighbours.
  if length(minneighbours)>1,
    maxnumedges = -1;
    for ii=minneighbours
      nn = EG(UU(ii),:);
      numedges = sum(sum(EG(nn,nn)));
      if numedges>maxnumedges
        maxnumedges = numedges;
        indx = ii;
      end
    end
  end
  uu = UU(indx); 
  order(ii) = uu;

  nodes = iintersect(neighbours(G,uu), UU); % look up neighbours in the partially filled-in graph
  nodes = iunion(nodes, uu); % the clique will always contain at least u
  G(nodes,nodes) = 1; % make them all connected to each other
  G = zerodiag(G);  
  eliminated(uu) = 1;
  
  exclude = 0;
  for c=1:length(cliques)
    if isubset(nodes,cliques{c}) % not maximal
      exclude = 1;
      break;
    end
  end
  if ~exclude
    cnum = length(cliques)+1;
    cliques{cnum} = nodes;
  end
end

tfill_ins = sparse(triu(max(0, G - MG), 1));
[I,J] = find(tfill_ins~=0);
fill_ins = num2cell([I,J],2);  
